/*
*
*  GroupUsersAction.java
*
*  SPECT: Simple Portal-Enhancing Configuration Tool
*  Copyright (C) 1999, 2000 Ray Ontko & Co.
*  http://www.ontko.com/
*
*  This program is a simple-to-use configuration tool for Brio.Portal
*  (formerly ReportMart).
*
*  "Brio.Portal" is a trademark of Brio Technology, http://www.brio.com/ .
* 
* 
*  This program is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by 
*  the Free Software Foundation; either version 2 of the License, or 
*  (at your option) any later version. 
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of 
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
*  Public License for more details. 
* 
*  A copy of the GPL is available online the GNU web site: 
* 
*    http://www.gnu.org/copyleft/gpl.html
* 
*/ 

import java.io.*;
import java.util.*;
import com.sqribe.rm.*;

public class GroupUsersAction extends BaseAction {

  private Repository repository;
  private static RMHashtable default_keywords;
  private RMHashtable keywords;
  private StreamTokenizer st;
  private int action;
  private int lineno;

  // Load default values into keyword hashtable
  static {
    default_keywords = new RMHashtable();
    RMProperty gp = new RMProperty("NAME","");
    default_keywords.put(gp.name, gp);
    gp = new RMProperty("USERS","");
    default_keywords.put(gp.name, gp);
  }

  public GroupUsersAction(Repository repository, int action, StreamTokenizer st,
         int lineno) throws IOException{
    super(action, st, lineno);
    this.repository = repository;
    this.action = action;
    this.type = "GROUPUSERS";
    keywords = (RMHashtable)default_keywords.clone();
    parse(default_keywords, keywords);
    doAction();
  }

  public int doAction() {
    switch (action) {
    case RMLoader.ADD :
      addGroupUsers();
      break;
    case RMLoader.REMOVE :
      removeGroupUsers();
      break;
    case RMLoader.SET :
      setGroupUsers();
    }
    return 0;
  }

  public void addGroupUsers() {
    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String name = rmp.value;
    User new_user = null;
    Group group;

    try {
      group = repository.getGroup(name);
    } 
    catch (ReportMartException rme) {
      RMLoader.logMsg("Couldn't find group \"" + rmp.value + "\".");
      return;
    }

      rmp = (RMProperty)keywords.get("USERS");
      String users[] = parseCommaSeparatedList(rmp.value);
      Vector add_users = new Vector();
      Vector v = new Vector();
      for (int i = 0; i < users.length; i++)
        try {
          add_users.addElement(repository.getUser(users[i]));
        }
        catch (ReportMartException rme) {
           RMLoader.logMsg("Invalid user \"" + users[i] + 
                         "\" (adding to group " 
                          + name + "), " + rme.getMessage());
        }
      try {
        QueryVector qv = group.listUsers();
        for (int i = 0; i < qv.size(); i++) 
          v.addElement(((User)qv.elementAt(i)));
      }
      catch (ReportMartException rme) {
           RMLoader.logMsg("Error getting user list for group " + name +
                           ", " + rme.getMessage() );
           return;
      }
    try {
      for (int i = 0; i < add_users.size(); i++)
        if (!v.contains(add_users.elementAt(i))) {
          new_user = (User)add_users.elementAt(i);
          group.addUser(new_user);
        }
      // group.update();
      RMLoader.logMsg("Successfully added users " + rmp.value + " to group " + 
                      name);
    } 
    catch (ReportMartException rme) {
      RMLoader.logMsg("Error adding user \"" + new_user + "\" to group " 
                       + name + ", " + rme.getMessage());
    }
  } 

  private void removeGroupUsers() {
    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String name = rmp.value;
    try {
      Group group = repository.getGroup(name);
      rmp = (RMProperty)keywords.get("USERS");
      String users[] = parseCommaSeparatedList(rmp.value);
      Vector rem_users = new Vector();
      for (int i = 0; i < users.length; i++)
        group.removeUser(repository.getUser(users[i]));
      // group.update();
      RMLoader.logMsg("Successfully removed users " + rmp.value + 
                      " from group " + name);
    }
    catch (ReportMartException rme) {
      RMLoader.logMsg("Error removing users " + rmp.value + " from group " +
                      name);
    }
  }

  private void setGroupUsers() {
    String users[] = null;
    Group group = null;

    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String name = rmp.value;
    try {
      group = repository.getGroup(name);

      // First, we remove all the the users from the groups 
      QueryVector qv = group.listUsers();
      User u;
      for (int i = 0; i < qv.size(); i++) {
        u = (User)qv.elementAt(i);
        group.removeUser(u);
        }
      //System.out.println("Done removing.");

    } 
    catch (ReportMartException rme) {
      RMLoader.logMsg("Error deleting users from group " + name + ": " +
                       rme.getMessage());
      return;
    }
     
    // Then we add the user into the groups specified.
    rmp = (RMProperty)keywords.get("USERS");
    users = parseCommaSeparatedList(rmp.value);
    try {
      for (int i = 0; i < users.length; i++)  
        group.addUser(repository.getUser(users[i]));
      // group.update();
      RMLoader.logMsg("Successfully set users  " + rmp.value + " for group " + 
                      name);
    }
    catch (ReportMartException rme) {
      RMLoader.logMsg("Error setting users " + rmp.value + " for group " +
                      name + ": " + rme.getMessage());
    }
  }
}
