/*
*
*  RMConfig.java
*
*  SPECT: Simple Portal-Enhancing Configuration Tool
*  Copyright (C) 1999, 2000 Ray Ontko & Co.
*  http://www.ontko.com/
*
*  This program is a simple-to-use configuration tool for Brio.Portal
*  (formerly ReportMart).
*
*  "Brio.Portal" is a trademark of Brio Technology, http://www.brio.com/ .
* 
* 
*  This program is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by 
*  the Free Software Foundation; either version 2 of the License, or 
*  (at your option) any later version. 
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of 
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
*  Public License for more details. 
* 
*  A copy of the GPL is available online the GNU web site: 
* 
*    http://www.gnu.org/copyleft/gpl.html
*
* 
*  This file contains methods related to the configuration settings
*  supported by SPECT.
* 
*/ 

//#FLAG API_V1.5_ONLY is DISABLED

import java.io.*;
import java.util.*;
import com.sqribe.rm.*;

public class RMConfig {

  public static final int ERROR = 0;
  public static final int WARNING = 1;
  public static final int IGNORE = 2;
  public static final int UPDATE = 3;
  public static final int ADD = 4;
  public static final int NEWVERSION = 5;
  public static final int REPLACE = 6;
  public static final int PURGE = 7;
  public static final int OLDEST = 8;
  public static final int NEWEST = 9;
  public static final int ALL = 10;
  public static final int OTHER = -1;

  public int when_preexists;
  public int when_missing;
  public int when_multiple_preexists;
  public int when_reloading_file;
  //#IF API_V1.5_ONLY DISABLED 
  public int when_maxversions_reached;
  //#END API_V1.5_ONLY
  public int when_multiple_exists;
  public int when_category_missing;
  public int when_file_missing;
  public int when_removing_from_default_group;
  public String default_category;
  public int when_ignored_keyword;
  public String good_file;
  public String bad_file;
  public String log_file;
  private Vector file_list;
  private int lineno;

  public RMConfig() {
    when_preexists                   = WARNING;
    when_missing                     = ERROR;
    when_multiple_preexists          = UPDATE;
//#IF API_V1.5_ONLY DISABLED 
    when_reloading_file              = NEWVERSION;
    when_maxversions_reached         = PURGE;
//#END API_V1.5_ONLY
//#IF API_V1.5_ONLY ENABLED 
//!    when_reloading_file              = REPLACE;
//#END API_V1.5_ONLY
    when_multiple_exists             = WARNING;
    when_category_missing            = ERROR;
    when_file_missing                = ERROR;
    when_removing_from_default_group = WARNING;
    default_category                 = "/";
    when_ignored_keyword             = WARNING;
    good_file                        = null;
    bad_file                         = null;
    log_file                         = null;
    file_list                        = new Vector();
  }

  public int parse(StreamTokenizer st) throws IOException {
    String option;
    int value_type;
    lineno = st.lineno();
    while (true) {
      st.nextToken();
      if (st.ttype == ';' || st.ttype == st.TT_EOF)
        return 0;
      option = st.sval;
      if (st.nextToken() != '=') {
        System.out.println("Error! \"=\" missing in config option " + option);
      }

      st.nextToken();
      if (st.ttype == st.TT_WORD || st.ttype == '"') {
        if (option.equalsIgnoreCase("WHEN-PREEXISTS")) {
          value_type = keywordValueToInt(st.sval);
          switch (value_type) {
            case ERROR :
            case WARNING :
            case IGNORE :
            case UPDATE :
              when_preexists = value_type;
              break;
            default :
              error("invalid value for config option when-preexists: " + 
                  st.sval);
          } // switch (value_type)
        } 
        else if (option.equalsIgnoreCase("WHEN-MISSING")) {
          value_type = keywordValueToInt(st.sval);
          switch (value_type) {
          case ERROR :
          case WARNING :
          case IGNORE :
            when_missing = value_type;
            break;
          default :
            error("invalid value for config option when-missing: " + 
                st.sval);
          } // switch (value_type)
        }
        else if (option.equalsIgnoreCase("WHEN-MULTIPLE-PREEXISTS")) {
          value_type = keywordValueToInt(st.sval);
          switch (value_type) {
            case ERROR :
            case WARNING :
            case IGNORE :
            case ADD :
            case REPLACE :
            case UPDATE :
              when_multiple_preexists = value_type;
              break;
            default :
              error("invalid value for config option " +
                    "when-multiple-preexists: " + st.sval);
          } // switch (value_type)
        }
        else if (option.equalsIgnoreCase("WHEN-MULTIPLE-EXISTS")) {
          value_type = keywordValueToInt(st.sval);
          switch (value_type) {
            case ERROR :
            case WARNING :
            case IGNORE :
            case OLDEST :
            case NEWEST :
            case ALL :
              when_multiple_exists = value_type;
              break;
            default :
              error("invalid value for config option when-multiple-exists: " + 
                  st.sval);
          } // switch (value_type)
        }
        else if (option.equalsIgnoreCase("WHEN-RELOADING-FILE")) {
          value_type = keywordValueToInt(st.sval);
          switch (value_type) {
//#IF API_V1.5_ONLY DISABLED 
            case NEWVERSION :
//#END API_V1.5_ONLY
            case REPLACE :
              when_reloading_file = value_type;
              break;
            default :
              error("invalid value for config option " +
                    "when-reloading-file: " + st.sval);
          } // switch (value_type)
        } 
//#IF API_V1.5_ONLY DISABLED 
        else if (option.equalsIgnoreCase("WHEN-MAXVERSIONS-REACHED")) {
          value_type = keywordValueToInt(st.sval);
          switch (value_type) {
            case ERROR :
            case WARNING :
            case IGNORE :
            case PURGE :
              when_maxversions_reached = value_type;
              break;
            default :
              error("invalid value for config option " +
                    "when-maxversions-reached: " + st.sval);
          } // switch (value_type)
        } 
//#END API_V1.5_ONLY
        else if (option.equalsIgnoreCase("WHEN-CATEGORY-MISSING")) {
          value_type = keywordValueToInt(st.sval);
          switch (value_type) {
            case ERROR :
            case WARNING :
            case IGNORE :
              when_category_missing = value_type;
              break;
            default :
              error("invalid value for config option when-category-missing: " + 
                  st.sval);
          }
        }
        else if (option.equalsIgnoreCase("WHEN-FILE-MISSING")) {
          value_type = keywordValueToInt(st.sval);
          switch (value_type) {
            case ERROR :
            case WARNING :
            case IGNORE :
            case UPDATE :
              when_file_missing = value_type;
              break;
            default :
              error("invalid value for config option when-file-missing: " + 
                  st.sval);
          }
        }
        else if (option.equalsIgnoreCase("WHEN-REMOVING-FROM-DEFAULT-GROUP")) {
          value_type = keywordValueToInt(st.sval);
          switch (value_type) {
            case ERROR :
            case WARNING :
            case IGNORE :
              when_removing_from_default_group = value_type;
              break;
            default :
              error("invalid value for config option " +
                    "when-removing-from-default-group: " + 
                  st.sval);
          }
        }
        else if (option.equalsIgnoreCase("DEFAULT-CATEGORY")) {
          if (st.ttype == st.TT_WORD || st.ttype == '"')
            default_category = st.sval;
        }
        else if (option.equalsIgnoreCase("WHEN-IGNORED-KEYWORD")) {
          value_type = keywordValueToInt(st.sval);
          switch (value_type) {
            case ERROR :
            case WARNING :
            case IGNORE :
              when_ignored_keyword = value_type;
              break;
            default:
              error("invalid value for config option when-ignored-keyword: " + 
                  st.sval);
          }
        }
        else if (option.equalsIgnoreCase("GOOD-FILE")) {
          if (st.ttype == st.TT_WORD || st.ttype == '"') {
            RMLoader.openGoodFile(st.sval);
          }
        }
        else if (option.equalsIgnoreCase("BAD-FILE")) {
          if (st.ttype == st.TT_WORD || st.ttype == '"') {
            RMLoader.openBadFile(st.sval);
          }
        }
        else if (option.equalsIgnoreCase("LOG-FILE")) {
          if (st.ttype == st.TT_WORD || st.ttype == '"') {
            RMLoader.openLogFile(st.sval);
          }
        }
        else {
          System.out.println("Invalid SETCONFIG option: " + option);
        }
      } // if (st.ttype == st.TT_WORD || st.ttype == '"') 
    } // while (true)
  } // private int parse()

  private int keywordValueToInt(String value) {
    if (value.equalsIgnoreCase("ERROR"))
      return ERROR;
    else if (value.equalsIgnoreCase("WARNING"))
      return WARNING;
    else if (value.equalsIgnoreCase("IGNORE"))
      return IGNORE;
    else if (value.equalsIgnoreCase("UPDATE"))
      return UPDATE;
    else if (value.equalsIgnoreCase("ADD"))
      return ADD;
//#IF API_V1.5_ONLY DISABLED 
    else if (value.equalsIgnoreCase("NEWVERSION")) 
      return NEWVERSION;
//#END API_V1.5_ONLY
    else if (value.equalsIgnoreCase("REPLACE"))
      return REPLACE;
//#IF API_V1.5_ONLY DISABLED 
    else if (value.equalsIgnoreCase("PURGE")) 
      return PURGE;
//#END API_V1.5_ONLY
    else if (value.equalsIgnoreCase("OLDEST"))
      return OLDEST;
    else if (value.equalsIgnoreCase("NEWEST"))
      return NEWEST;
    else if (value.equalsIgnoreCase("ALL"))
      return ALL;
    else return OTHER;
  }

  private void error(String msg) {
    RMLoader.logMsg("Error, " + lineno + ": " + msg);
  }
}
